package gui;

import java.awt.BorderLayout;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;

import model.Edge;
import model.Graph;
import model.Node;

public class DataForm extends JPanel {

	private static final String DELIMITER = ",";
	private static String path;
	public static List<Node> nodes = new ArrayList<Node>();
	public static List<Edge> edges = new ArrayList<Edge>();
	public static Graph graph;
	public static JTable table;
	public static JTextArea textArea;
	DefaultTableModel model;
	Object[] headers = { " ", "id", "name", "sex", "locale", "wallcount", "origin", "agerank" };
	int id;

	/**
	 * Metoda pro cteni CSV souboru
	 * 
	 * @return List<node>
	 */

	public Graph readFile(String path) {

		BufferedReader fileReader = null;

		try {
			String line = "";
			fileReader = new BufferedReader(new FileReader(path));

			while ((line = fileReader.readLine()) != null) {
				// vsechny sloupce v radku
				if (line.contains("nodedef>") || line.contains("edgedef>")) {
					// tak nic
				} else {
					String[] items = line.split(DELIMITER);
					if (items.length > 0) {
						if (items.length > 7) {
							// novy vrchol
							Node node = new Node(items[0], items[1], items[9], items[10], items[11], items[12],
									items[13]);
							node.setId(id);
							id++;
							nodes.add(node);
						}
						if (items.length == 4) {
							// nova hrana
							Edge edge = new Edge(Node.getNodeById(items[0], nodes), Node.getNodeById(items[1], nodes),
									items[2]);
							edges.add(edge);
						}
					}
				}
			}
			setInitialPosition(nodes);
			return new Graph(nodes, edges);
		} catch (Exception e) {
			System.out.println("Fail1");
			e.printStackTrace();
		} finally {
			try {
				fileReader.close();
			} catch (IOException e) {
				System.out.println("Fail2");
				e.printStackTrace();
			}
		}
		return null;
	}

	public DataForm() {
		if (fileChooser() == null) {
			System.exit(0);
		} else {
			getData();
			createTable();
		}
	}

	public void createTable() {
		// panel pro tabulku
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		table = new JTable(model);

		JScrollPane js = new JScrollPane(table);

		textArea = new JTextArea(10, 10);
		textArea.setEditable(false);

		// vypis vybraneho vrcholu + oznaceni ve scene
		table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

			@Override
			public void valueChanged(ListSelectionEvent e) {
				int row = table.getSelectedRow();
				for (Node node : nodes) {
					node.setIsSelected(false);
				}
				for (Edge edge : edges) {
					edge.setIsSelected(false);
				}
				textArea.setText("");
				for (int i = 0; i < table.getColumnCount(); i++) {
					if (i == 0) {
						textArea.append(table.getModel().getColumnName(i).toString() + "zznam:  ");
					} else {
						textArea.append(table.getModel().getColumnName(i).toString() + ":  ");
					}
					textArea.append(table.getModel().getValueAt(row, i).toString().replaceAll("\"", "") + "\n");
				}
				nodes.get(row).setIsSelected(true);
//				System.out.println(
//						"px: " + nodes.get(row).px + " py: " + nodes.get(row).py + " pz: " + nodes.get(row).pz);

				List<Edge> tmpEdges = Edge.getEdgesByNode(nodes.get(row), edges);
				for (Edge edge : tmpEdges) {
					edge.setIsSelected(true);
				}
				textArea.setCaretPosition(0);
			}
		});
		JScrollPane js2 = new JScrollPane(textArea);
		panel.add(js2, BorderLayout.NORTH);
		panel.add(js, BorderLayout.SOUTH);
		panel.setVisible(true);

		add(panel);
	}

	// Naplni tabulku daty z csv
	public void getData() {
		List<Node> nodesTmp = null;
		graph = null;
		try {
			if (path.length() > 0) {
				String extension = "";
				int i = path.lastIndexOf('.');
				if (i > 0) {
					extension = path.substring(i + 1);
					if (!extension.contains("gdf")) {
						JOptionPane.showMessageDialog(null, "Zvolte soubor formtu GDF", "Chyba! Formt "+extension+" nen podporovn", JOptionPane.ERROR_MESSAGE);
						System.exit(0);
					}
				}
				graph = readFile(path);
				nodesTmp = graph.getNodes();
			}
		} catch (Exception e) {
			// zadny vybrany soubor
		}

		model = new DefaultTableModel(new Object[0][0], headers) {
			@Override
			public boolean isCellEditable(int row, int column) {
				// nelze upravit bunky
				return false;
			}
		};
		// cislo radku v tabulce
		int line = 1;
		// naplnit tabulku
		try {
			for (Node node : nodesTmp) {
				Object[] o = new Object[8];
				o[0] = line++;
				o[1] = node.getIdS().replaceAll("\"", "");
				o[2] = node.getName().replaceAll("\"", "");
				o[3] = node.getSex().replaceAll("\"", "");
				o[4] = node.getLocale().replaceAll("\"", "");
				o[5] = node.getWallcount().replaceAll("\"", "");
				o[6] = node.getOrigin().replaceAll("\"", "");
				o[7] = node.getAgerank().replaceAll("\"", "");

				model.addRow(o);
			}
		} catch (Exception e) {
			// TODO: handle exception
		}
	}

	/**
	 * Vyber souboru
	 * 
	 * @return
	 */
	public static String fileChooser() {
		try {
			JFileChooser fileChooser = new JFileChooser();
			fileChooser.setCurrentDirectory(new File(System.getProperty("user.home")));
			int result = fileChooser.showOpenDialog(fileChooser);
			if (result == JFileChooser.APPROVE_OPTION) {
				File selectedFile = fileChooser.getSelectedFile();
				path = selectedFile.getAbsolutePath();
			}
		} catch (Exception e) {
			// TODO
		}
		return path;
	}

	public static void setInitialPosition(List<Node> nodes) {
		Random rand = new Random();
		int min = 0;
		int max = 100;
		for (Node n : nodes) {
			double randomNum1 = rand.nextInt((max - min) + 1) + min;
			double randomNum2 = rand.nextInt((max - min) + 1) + min;
			double randomNum3 = rand.nextInt((max - min) + 1) + min;

			n.setPx(randomNum1);
			n.setPy(randomNum2);
			n.setPz(randomNum3);
		}
	}
}
