package model;

import java.util.ArrayList;
import java.util.List;

public class FRLayout {
	public static List<Node> nodes = new ArrayList<Node>();
	public static List<Edge> edges = new ArrayList<Edge>();

	double k;
	double W = 2000;
	double H = 2000;
	double D = 2000;
	double area = W * H;

	int iteration = 0;
	int maxIteration;
	double temperature = 20;
	double gravity = 0.01;

	public void iteration(Graph g, int maxIteration) {
		nodes = g.getNodes();
		k = Math.sqrt(area) / (1 + nodes.size());

		if (iteration <= maxIteration) {
			// for (int i = 0; i < maxIteration; i++) {
			iteration++;
			// repulsive forces
			for (Node v : nodes) {
				v.dx = 0;
				v.dy = 0;
				v.dz = 0;
				for (Node u : nodes) {
					if (v != u) {
						if (v.px == u.px || v.py == u.py || v.pz == u.pz) {
							v.px += 0.1;
							v.py += 0.1;
							v.pz += 0.1;
						}
						double diffX = v.getPx() - u.getPx();
						double diffY = v.getPy() - u.getPy();
						double diffZ = v.getPz() - u.getPz();

						double diffM = Math.sqrt(diffX * diffX + diffY * diffY + diffZ * diffZ);

						v.dx = v.dx + (diffX / diffM) * fR(diffM);
						v.dy = v.dy + (diffY / diffM) * fR(diffM);
						v.dz = v.dz + (diffZ / diffM) * fR(diffM);
					}
				}
			}
			edges = g.getEdges();

			// attractive forces
			for (Edge e : edges) {
				// aby diff nebylo nikdy 0
				if (e.source.px == e.target.px || e.source.py == e.target.py || e.source.pz == e.target.pz) {
					e.source.px += 0.1;
					e.source.py += 0.1;
					e.source.pz += 0.1;
				}
				double diffX = e.source.getPx() - e.target.getPx();
				double diffY = e.source.getPy() - e.target.getPy();
				double diffZ = e.source.getPz() - e.target.getPz();

				double diffM = Math.sqrt(diffX * diffX + diffY * diffY + diffZ * diffZ);

				e.source.dx = e.source.dx - (diffX / diffM) * fA(diffM);
				e.source.dy = e.source.dy - (diffY / diffM) * fA(diffM);
				e.source.dz = e.source.dz - (diffZ / diffM) * fA(diffM);

				e.target.dx = e.target.dx + (diffX / diffM) * fA(diffM);
				e.target.dy = e.target.dy + (diffY / diffM) * fA(diffM);
				e.target.dz = e.target.dz + (diffZ / diffM) * fA(diffM);
			}
			// omezit pro frame
			for (Node v1 : nodes) {
				double disp = Math.sqrt(v1.dx * v1.dx + v1.dy * v1.dy + v1.dz * v1.dz);

				// gravitace
				double gForce = k * gravity;				

				v1.setPx((v1.px + (v1.dx / disp) * Math.min(v1.dx, temperature)));
				v1.setPy((v1.py + (v1.dy / disp) * Math.min(v1.dy, temperature)));
				v1.setPz((v1.pz + (v1.dz / disp) * Math.min(v1.dz, temperature)));

				v1.setPx(Math.min(W, Math.max(-W, v1.getPx())) - gForce);
				v1.setPy(Math.min(H, Math.max(-H, v1.getPy())) - gForce);
				v1.setPz(Math.min(D, Math.max(-D, v1.getPz())) - gForce);
			}

			temperature *= 0.99;
		} else {
		}
	}

	public double fA(double x) {
		return (x * x) / (k);
	}

	public double fR(double x) {
		return -(k * k) / (x);
	}
}
